package com.ycwl.basic.controller.mobile;

import com.ycwl.basic.model.mobile.goods.GoodsDetailVO;
import com.ycwl.basic.model.mobile.goods.GoodsPriceQueryReq;
import com.ycwl.basic.model.mobile.goods.GoodsReqQuery;
import com.ycwl.basic.model.mobile.goods.GoodsPageVO;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.service.mobile.GoodsService;
import com.ycwl.basic.utils.ApiResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * @Author：longbinbin
 * @Date：2024/12/5 14:49
 * 成片（video）和源素材（source）都可作为商品，订单中可购买多个
 */
@RestController
@RequestMapping("/api/mobile/goods/v1")
@Api(tags = "商品相关接口")
public class AppGoodsController {

    @Autowired
    private GoodsService goodsService;

    @ApiOperation("商品列表")
    @PostMapping("/goodsList")
    public ApiResponse<List<GoodsPageVO>> goodsList(@RequestBody GoodsReqQuery query) {
        return goodsService.goodsList(query);
    }

    @ApiOperation("源素材(原片/照片)商品列表")
    @PostMapping("/sourceGoodsList")
    public ApiResponse<List<GoodsDetailVO>> sourceGoodsList(@RequestBody GoodsReqQuery query) {
        return goodsService.sourceGoodsList(query);
    }

    @ApiOperation("成片vlog商品详情")
    @GetMapping("/getVideoGoodsDetail/{goodId}")
    public ApiResponse<GoodsDetailVO> videoGoodsDetail(@PathVariable("goodId") Long goodsId) {
        return goodsService.videoGoodsDetail(goodsId);
    }

    @ApiOperation("查询价格")
    @PostMapping("/queryPrice")
    public ApiResponse<BigDecimal> queryPrice(@RequestBody GoodsPriceQueryReq queryPriceData) {
        return goodsService.queryPrice(queryPriceData);
    }
}
